function varargout = diehl_dyn(x,ubar,w_n,wat)
%% DESCRIPTION
% Diehl_dyn - Caculates the state derivatives and the output variables to
% simulate the point-mass model.
%
% Syntax:  output = function_name(x,ubar,w_n,wat)
%
% Inputs:
%    x - current states: [theta, dtheta, phi, dphi, int_T].'
%    ubar - difference in line lengths
%    w_n - stochastic part of wind speed
%    wat - either 'y' or 'f', depending on whether you output/dynamic equ.
%
% Outputs:
%    output - if wat = 'y', is the vector: [alpha; Waspeed; T; gamma; psi; E_eff; eta]
%    output - if wat = 'f', is the vector: [dtheta, ddtheta, dphi, ddphi, T].'
%
% Files required: calcsaved1.m, calcsaved2.m
%
% See also: sens_calc.m

% Author: Sean Costello
% Automatic Control Lab, EPFL, Lausanne, Switzerland
% email: sean.costello@epfl.ch
% Website: http://people.epfl.ch/sean.costello
% March 2014; Last revision: 21-March-2014

%% ------------- CODE --------------
global P;

% unpack the states (they must have the same names as used in sens_calc.m)
theta = x(1);
dtheta = x(2);
phi = x(3);
dphi = x(4);

% rename the physical variables (they must have the same names as used in sens_calc.m)
r = P.r;
m = P.m;
g = P.g;
rho = P.rho;
d = P.d;
A = P.A;

% Load co-ordinate frame calculations from sens_calc.m (p, dp and L)
calcsaved1;

if(p(3)<=0)
    error('Kite hit the ground')
end

%% calculate forces numerically
Fg = [0; -0; -m*g]; % gravity
Fg_L = L\Fg; % gravity in L frame

% effective wind
% Wind power law from: Cristina Archer 2014, Airborne Wind Energy, Springer
% The windspeed is zero at z=0! 
w = (P.w_ref+w_n)*(p(3)/P.z_ref)^P.a; %p(3) = z
W = w*(cos(P.chi)*[1;0;0]+sin(P.chi)*[0;1;0]); %wind vector, vw = windspeed (SHEAR)
Wa = W - dp; % apparent wind vector
Wa_L = L\Wa; %.. in L
Waspeed = norm(Wa);  %effective windspeed

% get basis for L2 in L1 (for going from L2 to L1): x_L = L2_L*x_L2
e_down_L = [0; 0; 1];
Wap_L = [Wa_L(1:2);0]; %apparent wind projected onto tangent plane
e_f = -Wap_L/norm(Wap_L); %unit vector in opposite direction to Wap
e_0_L = cross(e_down_L,e_f); %no apparent wind in e_0 direction
L2_L = [e_f, e_0_L, e_down_L]; % basis vectors for L2, expressed in L

%rotation (in L2) around e_f by mu (remark: mu is called psi in DiehlThesis)
%pi/2 - mu is the angle the pitch axis makes with lines due to steering deflection. 
mu = asin(-ubar/d); %u is the DIFFERENCE in line lengths, the minus is to match with skysails model
R1 = [1 0 0; 0 cos(mu) -sin(mu); 0 sin(mu) cos(mu)]; 
%apparent wind in L2
Wa_L2 = L2_L\Wa_L; 
%calculate eta such that e_pitch is perp to Wa
eta = asin(tan(mu)*(Wa_L2(3)/Wa_L2(1))); 
if(~isreal(eta)), 
    error('too large an input: cannot point kite into effective wind!')
end
% a real eta may not exist if the input is too large!
% this can happen when the apparent wind and the line are nearly aligned.
% In reality the kite would begin to sideslip = realistic to assume you
% lose control and crash in this situation!

R2 = [cos(eta) -sin(eta) 0; sin(eta) cos(eta) 0; 0 0 1]; %rotation (in L2) around e_down_L2 by eta

e_pitch_L = L2_L*R2*R1*[0; 1; 0]; %R2*R1*e_0_L2 in L
e_roll_L = L2_L*R2*R1*[1; 0; 0]; %R2*R1*e_w_L2 in L

%are the assumptions verified?
residual1 = e_pitch_L.'*e_down_L -sin(mu);  %correct line shortening?
residual2 = e_pitch_L.'*Wa_L;               %infinite tail assumption?
if(any(abs([residual1, residual2])>1e-10))
    display('kite orientation assumptions not satisfied'), keyboard
end

zw_L = cross(Wa_L/norm(Wa_L),e_pitch_L); %unit vector in lift vector direction
xw_L = Wa_L/norm(Wa_L); %unit vector in drag vector direction

alpha = acos(e_roll_L'*(-xw_L)); %this is the AoA w.r.t. the kite's roll axis

% L and D curves
CD = P.CD0 + P.CD2*alpha^2; 
CL = P.CL0 + P.CL1*alpha;

E = CL/CD; %glide ratio
E_eff = (CL/CD)*cos(mu); %effective glide ratio for tension generation, E_eff < E due to turning
Fa_L = (0.5*rho*A*(Wa_L'*Wa_L))*(CL*zw_L + CD*xw_L); %aerodynamic force

% now get forces in L basis
Ftheta = Fa_L(1) + Fg_L(1);
Fphi = Fa_L(2) + Fg_L(2);

% evaluate dtheta, ddphi, Fdown using newton's law results
calcsaved2;
T = Fdown - (Fa_L(3)+Fg_L(3)) ; %tether force

% get velocity angle
gamma = atan2(dphi*sin(theta),dtheta);

% get heading angle as angle from e_theta to roll vector, going CLOCKWISE when looking from origin
% definition is the same as heading angle in Skysails, but its dynamics
% will of course be different as there is direct feedthrough between u and it
psi = atan2(-e_roll_L(2),e_roll_L(1));

% decide whether to output the 'states' of the point-mass simulator, or the
% outputs
switch wat
    case 'f'
        varargout{1} = [dtheta, ddtheta, dphi, ddphi, T].'; 
    case 'y'
        varargout{1} = [alpha; Waspeed; T; gamma; psi; E_eff; eta];
end

end