function fagiano_pathplanner_SF(block)
%% DESCRIPTION
% fagiano_pathplanner_SF - A level-2 S-Function implementation of the
% path planning controller from: 
% Fagiano, L., Zgraggen, A., Morari, M., Khammash, M., 2013. Automatic
% crosswind flight of tethered wings for airborne wind energy: Modeling, control
% design, and experimental results. Control Systems Technology, IEEE
% Transactions on, preprints
% A super-simple algorithm to alternate the kite between 2 points.
%
% Syntax:  standard level-2 S-function syntax
%
% Files required: none

% Author: Sean Costello
% Automatic Control Lab, EPFL, Lausanne, Switzerland
% email: sean.costello@epfl.ch
% Website: http://people.epfl.ch/sean.costello
% March 2014; Last revision: 21-March-2014

%% ------------- CODE -------------- 

setup(block);
  
function setup(block)
  global C; % controller parameters
  
  % Register number of dialog parameters   
  block.NumDialogPrms = 0; 
 
  % Register number of input and output ports
  block.NumInputPorts  = 3;
  block.NumOutputPorts = 1;
 
  % Setup functional port properties to dynamically inherited.
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
 
  %overwrite some of the inherited settings
  for i = 1:3
      block.InputPort(i).Dimensions         = 1;
      block.InputPort(i).Complexity         = 'Real';
      block.InputPort(i).DirectFeedthrough  = true; 
  end 
  block.OutputPort(1).Dimensions        = 1;   
  block.OutputPort(1).Complexity        = 'Real';
  
  % Set block sample time (0 = continuous)
  block.SampleTimes = [C.T_s 0];
   
  % Setup continuous states
  %block.NumContStates = 1;

  % Set the block simStateCompliance to default (i.e., same as a built-in block)
  block.SimStateCompliance = 'DefaultSimState';
 
  % Register methods
%   block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
%   block.RegBlockMethod('Derivatives',             @Derivative);  
  block.RegBlockMethod('SetInputPortSamplingMode', @SetInpPortFrameData);
  block.RegBlockMethod('PostPropagationSetup', @DoPostPropSetup);
  block.RegBlockMethod('Start', @Start);
  block.RegBlockMethod('Update', @Update);
  
function SetInpPortFrameData(block, idx, fd)  
% SetInpPortFrameData is necessary if there are multiple inputs or outputs
  block.InputPort(idx).SamplingMode = 0;    %not for frame data
  block.OutputPort(1).SamplingMode  = 0;

function DoPostPropSetup(block)
  block.NumDworks = 1;  % discrete state vector  
  block.Dwork(1).Name            = 'xd';
  block.Dwork(1).Dimensions      = 1;
  block.Dwork(1).DatatypeID      = 0;      % double
  block.Dwork(1).Complexity      = 'Real'; % real
  block.Dwork(1).UsedAsDiscState = true;
  
function Start(block)
  global C;
  %intialize discrete state
  block.Dwork(1).Data = C.phiM; 
  
function Output(block)
    global C;    
    phiT = block.Dwork(1).Data;
    theta = block.InputPort(1).Data; 
    phi = block.InputPort(2).Data;
    % minus the gamma version in Fagiano's paper, as we are using psi ~= -gamma   
    psi_ref = -atan2(((phiT - phi)*sin(theta)),(C.thetaT - theta)); % point towards target
    block.OutputPort(1).Data = psi_ref;
    
function Update(block)
    global C;   
    phi = block.InputPort(2).Data;   
    lastphiT = block.Dwork(1).Data;    
    if phi>C.phiP   % if phi crosses target value, switch to the other target point
        phiT = C.phiM;
    elseif phi<C.phiM
        phiT = C.phiP;
    else
        phiT = lastphiT;    % no crossing detected -> keep last target point
    end
    block.Dwork(1).Data = phiT; % the target value of phi is the state
 
   
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    