%% DESCRIPTION
% main.m - the top-level script that should be called to simulate the kite
% benchmark in closed loop with our implementation of the controller proposed by 
% Fagiano, L., Zgraggen, A., Morari, M., Khammash, M., 2013. Automatic
% crosswind flight of tethered wings for airborne wind energy: Modeling, control
% design, and experimental results. Control Systems Technology, IEEE
% Transactions on, preprints
%
% Files required: diehl_dyn.m, plant_params.m, simreality.mdl
%

% Author: Sean Costello
% Automatic Control Lab, EPFL, Lausanne, Switzerland
% email: sean.costello@epfl.ch
% Website: http://people.epfl.ch/sean.costello
% March 2014; Last revision: 7-April-2015

%% ------------- CODE --------------
close all; clear all; clc;

global P C; % P and C are structures containing the plant and controller parameters (resp.) 

% load plant parameters
plant_params;
% x0 = intial states, x: [theta, dtheta, phi, dphi, int_T]
P.x0 = [P.theta_0, P.dtheta_0, P.phi_0, P.dphi_0, 0];

% simulation parameters
simp.tf = 200; %final time

% controller params
C.T_s = P.T_s;  %sampling period, use that of plant
C.K = 8;      % proportional gain in psi controller
C.I = 2.5;      % integral gain in psi controller
C.u_max = P.u_max; %maximum value the controller will assign to u

wT = 120; %distance (straight line) between the two target points
zT = 110; %altitude of the target points

% get target points in spherical co-ords
C.phiP = atan((wT/2)/zT);
C.phiM = -C.phiP;
C.thetaT = asin((((wT/2)^2 + zT^2)^.5)/P.r);

% theta filter for controller
fc = 1; %cut-off frequency (Hz)
fc_scaled = fc/(0.5/C.T_s); %scale w.r.t nyquist f 
[C.Btheta,C.Atheta]=butter(2,fc_scaled);

% phi filter for controller
fc = 1; %cut-off frequency (Hz)
fc_scaled = fc/(0.5/C.T_s); %scale w.r.t nyquist f
[C.Bphi,C.Aphi]=butter(2,fc_scaled);

% psi filter for controller
fc = 1; %cut-off frequency (Hz)
fc_scaled = fc/(0.5/C.T_s); %scale w.r.t nyquist f
[C.Bpsi,C.Apsi]=butter(2,fc_scaled);

% psir (reference signal) filter for controller
fc = 1; %cut-off frequency (Hz)
fc_scaled = fc/(0.5/C.T_s); %scale w.r.t nyquist f
[C.Bpsir,C.Apsir]=butter(2,fc_scaled);

% put simulation parameters in special simulink structure
simpar.AbsTol   = '1e-5';
simpar.StartTime = '0';
simpar.StopTime = num2str(simp.tf);
simpar.RelTol = '1e-5';
simpar.SrcWorkspace = 'current';

%% call simulink
tic
simout = sim('simreality', simpar);
%simout = sim('simreality_bangbang', simpar);
display(['The simulation took ', num2str(toc), sprintf(' s \n------------\n')]);
simlog = simout.get('simlog');

%% results
% print out average T obtained. Note that although yN is the system's 
% noisy outputs (to be used by the control algorithm), the integral of T is
% noise-free, a 'perfect' measurement.
display([sprintf('\n------------\n T, averaged over the period from 0 s to '),...
    simpar.StopTime, ' s, is ',...
    num2str(simlog.yN.int_T.Data(end)/(1e3*str2num(simpar.StopTime))), sprintf(' kN \n------------\n')])

% plot the kite's real heading angle, and compare it with the measured value
figure('Name', 'Heading Angle')
plot(simlog.ysim.psi.Time, simlog.ysim.psi.Data,'b')
hold on;
plot(simlog.yN.psiN.Time, simlog.yN.psiN.Data,'r--')
legend('plant value', 'measured value')
xlabel('time (s)'), ylabel('heading angle, \psi (rad)')

% plot the kite's altitude
figure('Name', 'Altitude vs. Time')
plot(simlog.ysim.theta.Time, P.r*sin(simlog.ysim.theta.Data).*cos(simlog.ysim.phi.Data))
xlabel('time (s)'), ylabel('kite altitude, z (m)')

% plot the kite's path on the quarter sphere
figure('Name','Kite Path (hollow dot = intial point, solid dot = target points)');
my3Dplotter(simlog.ysim.theta.Data,simlog.ysim.phi.Data,'b');
my3Dplotter(C.thetaT,C.phiM,'r*');
my3Dplotter(C.thetaT,C.phiP,'r*');

%%
break
main_further;    % script with more code, not distributed









