function pointmass_SF(block)
%% DESCRIPTION
% pointmass_SF - formulation required to integrate the point-mass model
% into a simulink environment. Basically just calls diehl_dyn.m, the rest
% of the code is just to set block properties: the number of inputs, states, outputs,
% sample-times, etc..
%
% Syntax:  standard level-2 S-function syntax
%
% Files required: diehl_dyn.m
%
% See also: OTHER_FUNCTION_NAME1,  OTHER_FUNCTION_NAME2

% Author: Sean Costello
% Automatic Control Lab, EPFL, Lausanne, Switzerland
% email: sean.costello@epfl.ch
% Website: http://people.epfl.ch/sean.costello
% March 2014; Last revision: 15-April-2014

%% ------------- CODE --------------

setup(block);
  
function setup(block)

  % Register number of dialog parameters   
  block.NumDialogPrms = 0; 
 
  % Register number of input and output ports
  block.NumInputPorts  = 2;
  block.NumOutputPorts = 2;
 
  % Setup functional port properties to dynamically inherited.
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
 
  % overwrite some of the inherited settings
  block.InputPort(1).Dimensions        = 1;
  % DirectFeedthrough=true can give algebraic loop warnings 
  % when S-function is part of a control loop, even if the controlled variable does not
  % have direct feedthrough..
  block.InputPort(1).DirectFeedthrough = true; 
  block.InputPort(2).Dimensions        = 1;
  block.InputPort(2).DirectFeedthrough = true;
  
  block.OutputPort(1).Dimensions       = 5; 
  block.OutputPort(2).Dimensions       = 7; 
  
  % Set block sample time to continuous (number of states or outputs?)
  block.SampleTimes = [0 0];
   
  % Setup continuous states
  block.NumContStates = 5;
 
  % Set the block simStateCompliance to default (i.e., same as a built-in block)
  block.SimStateCompliance = 'DefaultSimState';
 
  % Register methods
  block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('Derivatives',             @Derivative);  
  block.RegBlockMethod('SetInputPortSamplingMode', @SetInpPortFrameData);
 
function SetInpPortFrameData(block, idx, fd)  
% SetInpPortFrameData is necessary if there are multiple inputs or outputs
  block.InputPort(idx).SamplingMode = 0;    %not for frame data
  block.OutputPort(1).SamplingMode  = 0;
  block.OutputPort(2).SamplingMode  = 0;
   
  
function InitConditions(block) % Initialize states
    global P waserror;
    waserror = 'false';
    block.ContStates.Data = [P.x0];
   
function Output(block)
    global waserror;
    x = block.ContStates.Data;      % current states 
    u = block.InputPort(1).Data;    % input 1 is the steering deflection
    w_n = block.InputPort(2).Data;  % input 2 is the wind-speed noise    
    try
        y_sim = diehl_dyn(x,u,w_n,'y'); % call dynamics
    catch myerror
        if strcmp(waserror,'false') % error just occurred
            set_param(bdroot,'SimulationCommand','stop'); %stop the simulation
            display(sprintf(['Simulation stopped because: %s \n',... %print message
                'At simulation time: %0.3f sec \nFile: %s \nLine: %d \n---------\n'],...
                myerror.message, block.currenttime, eval('myerror.stack.file'),...
                eval('myerror.stack.line')));
            waserror = 'true';  % simulink may continue for a few steps, 
                                % but error has been treated
        end
        y_sim = zeros(block.OutputPort(2).Dimensions,1);
    end
    
    block.OutputPort(1).Data = x;   % send out states
    block.OutputPort(2).Data = y_sim;   % send out y_sim
 
function Derivative(block)  
    x = block.ContStates.Data;      
    u = block.InputPort(1).Data;    
    w_n = block.InputPort(2).Data; 
    try
        dx = diehl_dyn(x,u,w_n,'f'); % dx = [dtheta, ddtheta, dphi, ddphi, T];
    catch 
        dx = zeros(block.NumContStates,1); 
    end
    block.Derivatives.Data = dx;
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    