%% DESCRIPTION
% This script performs some of the calculations for the point-mass model
% symbolically, and saves the results to .m files
%
% Files required: none
% Files produced: calcsaved1.m, calcsaved2.m

% Author: Sean Costello
% Automatic Control Lab, EPFL, Lausanne, Switzerland
% email: sean.costello@epfl.ch
% Website: http://people.epfl.ch/sean.costello
% March 2014; Last revision: 21-March-2014

%% ------------- CODE --------------
clc; clear all; close all;

syms theta phi r dtheta dphi dr ddtheta ddphi ddr x y z Ftheta Fphi Fdown real; % variables
syms m g real; % physical parameters

% get vectors of spherical coords and their derivatives
s = [theta; phi; r]; 
ds = [dtheta; dphi; dr];
dds = [ddtheta; ddphi; ddr];

% get kite position in general system, G, and its derivatives
p = r*[cos(theta);...
       sin(theta)*sin(phi);...
       sin(theta)*cos(phi)]; %p(theta,phi,r)
   
dp_ds = jacobian(p,s); % [dp/dtheta, dp/dphi, dp/dr]
dp = dp_ds*ds;
ddp= jacobian(dp,s)*ds + jacobian(dp,ds)*dds;

% set line length constant in velocity and acceleration exp.
dp = subs(dp,[dr ddr],[0 0]); 
ddp = subs(ddp,[dr ddr],[0 0]);

% get L: local system, orthonormal (RIGHT HAND basis)
e_theta = dp_ds(:,1)/r; 
e_phi = dp_ds(:,2)/(r*sin(theta)); 
e_down = -dp_ds(:,3); % -e_r
L = [e_theta, e_phi, e_down]; %basis for local sys, note: using -e_r to make it right hand!

% get acceleration in L: ddp = e_L*ddp_L => ddp_L = e_L\ddp
ddp_L = simplify(L\ddp); %acceleration in L basis
dp_L = simplify(L\dp); %velocity in L basis

% write newtons law
newton = m*ddp_L==[Ftheta; Fphi; Fdown]; %remember, Fdown is towards origin!
% solve newton's law for the acceleration of the spherical coords, and for
% the total force in the 'down' direction
[ddtheta,ddphi,Fdown] = solve(newton, ddtheta, ddphi, Fdown,...
            'IgnoreAnalyticConstraints',true,'Real',true);
        
%% saving
% When simulating the system you will need [L, p, dp] to calculate the forces
% numerically [SAVE THESE SEPARATELY in calcsaved1.m]
% Once the numerical values of these forces have been evaluated you can use 
% the symbolic expression's for [ddtheta, ddphi, Fdown] obtained from newton's
% law[SAVE in calcsaved2.m ]

% The following code is just to save the expressions to .m files...

% Definition of the file name
File_name = 'calcsaved1.m';

% List of variables to save
List_of_variables = {'L','p','dp'};

% Saving algorithm
fid = fopen(File_name,'wt');
index_equ = 1;
for k = 1:length(List_of_variables)
    Var_name = List_of_variables{k};
    size_var = size(eval(Var_name));
    for i = 1:size_var(1)
        for j = 1:size_var(2)
            Var_name_ij = strcat(Var_name,'(',num2str(i),',',num2str(j),')');
            Equation = strcat(Var_name_ij,' = ',char(eval(Var_name_ij)),';');
            fprintf(fid,'%s \n',Equation);
        end
    end
end
fclose(fid);


% Definition of the file name
File_name = 'calcsaved2.m';

% List of variables to save
List_of_variables = {'ddtheta','ddphi','Fdown'};

% Saving algorithm
fid = fopen(File_name,'wt');
index_equ = 1;
for k = 1:length(List_of_variables)
    Var_name = List_of_variables{k};
    size_var = size(eval(Var_name));
    for i = 1:size_var(1)
        for j = 1:size_var(2)
            Var_name_ij = strcat(Var_name,'(',num2str(i),',',num2str(j),')');
            Equation = strcat(Var_name_ij,' = ',char(eval(Var_name_ij)),';');
            fprintf(fid,'%s \n',Equation);
        end
    end
end
fclose(fid);


